import { ComponentType } from 'react';
import { CSSProperties } from 'react';
import { DependencyList } from 'react';
import { ForwardedRef } from 'react';
import { FunctionComponent } from 'react';
import { ReactElement } from 'react';
import { RefObject } from 'react';

/**
 * Make annotation from the markup
 */
export declare function annotate(markup: Markup, label: string, value?: string): string;

export declare type ConfiguredMarkedInput<T> = FunctionComponent<MarkedInputProps<T>>;

/**
 * Create the configured MarkedInput component.
 */
export declare function createMarkedInput<T = MarkStruct>(configs: Omit<MarkedInputProps<T>, 'value' | 'onChange'>): ConfiguredMarkedInput<T>;

/**
 * Transform the annotated text to the another text
 */
export declare function denote(value: string, callback: (mark: MarkMatch) => string, ...markups: Markup[]): string;

export declare type label = `${string}${PLACEHOLDER.LABEL}${string}`;

export declare type Listener<T = any> = (e: T) => void;

export declare const MarkedInput: MarkedInputComponent;

export declare interface MarkedInputComponent {
    <T = MarkStruct>(props: MarkedInputProps<T>): JSX.Element | null;
    displayName?: string;
}

export declare interface MarkedInputHandler {
    /**
     * Container element
     */
    readonly container: HTMLDivElement | null;
    /**
     * Overlay element if exists
     */
    readonly overlay: HTMLElement | null;
    focus(): void;
}

export declare interface MarkedInputProps<T = MarkStruct> {
    /**
     * Ref to handler
     */
    ref?: ForwardedRef<MarkedInputHandler>;
    /**
     * Annotated text with markups for mark
     */
    value: string;
    /**
     * Change event
     */
    onChange: (value: string) => void;
    /**
     * Component that used for render markups
     */
    Mark?: ComponentType<T>;
    /**
     * Component that used for render overlays such as suggestions, mentions, autocomplete, modal, tooltip and etc.
     */
    Overlay?: ComponentType;
    /**
     * Prevents from changing the value
     */
    readOnly?: boolean;
    /**
     * Passed options for configure
     * @default One option with default values is used
     */
    options?: Option_2<T>[];
    /**
     * Additional classes
     */
    className?: string;
    /**
     * Additional style
     */
    style?: CSSProperties;
    /**
     * Forward props to internal components via react elements
     * @param div - to container
     */
    children?: ReactElement | ReactElement[];
    /**
     * Triggering events for overlay
     * @default 'change'
     */
    trigger?: OverlayTrigger;
}

export declare interface MarkHandler<T> extends MarkStruct {
    /**
     * MarkStruct ref. Used for focusing and key handling operations.
     */
    ref: RefObject<T>;
    /**
     * Change mark.
     * @options.silent doesn't change itself label and value, only pass change event.
     */
    change: (props: MarkStruct, options?: {
        silent: boolean;
    }) => void;
    /**
     * Remove itself.
     */
    remove: () => void;
    /**
     * Passed the readOnly prop value
     */
    readOnly?: boolean;
}

export declare interface MarkMatch extends MarkStruct {
    annotation: string;
    input: string;
    /**
     * Start position of a overlayMatch
     */
    index: number;
    optionIndex: number;
}

export declare interface MarkStruct {
    label: string;
    value?: string;
}

export declare type Markup = `${label}${value}` | `${label}`;

declare interface Option_2<T = Record<string, any>> {
    /**
     * Template string instead of which the mark is rendered.
     * Must contain placeholders: `__label__` and optional `__value__`
     * @default "@[__label__](__value__)"
     */
    markup?: Markup;
    /**
     * Sequence of symbols for calling the overlay.
     * @default "@"
     */
    trigger?: string;
    /**
     * Data for an overlay component. By default, it is suggestions.
     */
    data?: string[];
    /**
     * Function to initialize props for the mark component. Gets arguments from found markup
     */
    initMark?: (props: MarkStruct) => T;
}
export { Option_2 as Option }

export declare interface OverlayHandler {
    /**
     * Style with caret absolute position. Used for placing an overlay.
     */
    style: {
        left: number;
        top: number;
    };
    /**
     * Used for close overlay.
     */
    close: () => void;
    /**
     * Used for insert an annotation instead a triggered value.
     */
    select: (value: MarkStruct) => void;
    /**
     * Overlay match details
     */
    match: OverlayMatch;
    ref: RefObject<HTMLElement>;
}

export declare type OverlayMatch = {
    /**
     * Found value via a overlayMatch
     */
    value: string;
    /**
     * Triggered value
     */
    source: string;
    /**
     * Piece of text, in which was a overlayMatch
     */
    span: string;
    /**
     * Html element, in which was a overlayMatch
     */
    node: Node;
    /**
     * Start position of a overlayMatch
     */
    index: number;
    /**
     * OverlayMatch's option
     */
    option: Option_2;
};

export declare type OverlayTrigger = Array<'change' | 'selectionChange'> | 'change' | 'selectionChange' | 'none';

export declare enum PLACEHOLDER {
    LABEL = "__label__",
    VALUE = "__value__"
}

export declare enum SystemEvent {
    Change = 0,
    Delete = 1,
    CheckTrigger = 2,
    ClearTrigger = 3,
    Trigger = 4,
    Select = 5,
    State = 6
}

export declare function useListener(type: SystemEvent, listener: Listener, deps?: DependencyList): void;

export declare function useListener<K extends keyof HTMLElementEventMap>(type: K, listener: Listener<HTMLElementEventMap[K]>, deps?: DependencyList): void;

export declare const useMark: <T extends HTMLElement = HTMLElement>() => MarkHandler<T>;

export declare function useOverlay(): OverlayHandler;

export declare type value = `${string}${PLACEHOLDER.VALUE}${string}`;

export { }
